@@ -27,8 +27,8 @@ import static org.junit.Assert.fail;
 
 import java.io.IOException;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.ContentSummary;
 import org.apache.hadoop.fs.FSDataInputStream;
@@ -45,13 +45,15 @@ import org.apache.hadoop.hdfs.server.protocol.NamenodeProtocols;
 import org.apache.hadoop.ipc.RemoteException;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.hadoop.test.LambdaTestUtils;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
 public class TestHDFSConcat {
-  public static final Log LOG = LogFactory.getLog(TestHDFSConcat.class);
+  public static final Logger LOG =
+      LoggerFactory.getLogger(TestHDFSConcat.class);
 
   private static final short REPL_FACTOR = 2;
   
@@ -84,10 +86,12 @@ public class TestHDFSConcat {
   public void shutDownCluster() throws IOException {
     if(dfs != null) {
       dfs.close();
+      dfs = null;
     }
     if(cluster != null) {
       cluster.shutdownDataNodes();
       cluster.shutdown();
+      cluster = null;
     }
   }
   
@@ -111,18 +115,21 @@ public class TestHDFSConcat {
     long trgBlocks = nn.getBlockLocations(trg, 0, trgLen).locatedBlockCount();
        
     Path [] files = new Path[numFiles];
-    byte [] [] bytes = new byte [numFiles][(int)fileLen];
+    byte[][] bytes = new byte[numFiles + 1][(int) fileLen];
     LocatedBlocks [] lblocks = new LocatedBlocks[numFiles];
     long [] lens = new long [numFiles];
     
-    
+    stm = dfs.open(trgPath);
+    stm.readFully(0, bytes[0]);
+    stm.close();
     int i;
     for(i=0; i<files.length; i++) {
       files[i] = new Path("/file"+i);
       Path path = files[i];
       System.out.println("Creating file " + path);
-      DFSTestUtil.createFile(dfs, path, fileLen, REPL_FACTOR, 1);
-    
+
+      // make files with different content
+      DFSTestUtil.createFile(dfs, path, fileLen, REPL_FACTOR, i);
       fStatus = nn.getFileInfo(path.toUri().getPath());
       lens[i] = fStatus.getLen();
       assertEquals(trgLen, lens[i]); // file of the same length.
@@ -131,7 +138,7 @@ public class TestHDFSConcat {
       
       //read the file
       stm = dfs.open(path);
-      stm.readFully(0, bytes[i]);
+      stm.readFully(0, bytes[i + 1]);
       //bytes[i][10] = 10;
       stm.close();
     }
@@ -153,6 +160,17 @@ public class TestHDFSConcat {
     // check count update
     ContentSummary cBefore = dfs.getContentSummary(trgPath.getParent());
     
+    // resort file array, make INode id not sorted.
+    for (int j = 0; j < files.length / 2; j++) {
+      Path tempPath = files[j];
+      files[j] = files[files.length - 1 - j];
+      files[files.length - 1 - j] = tempPath;
+
+      byte[] tempBytes = bytes[1 + j];
+      bytes[1 + j] = bytes[files.length - 1 - j + 1];
+      bytes[files.length - 1 - j + 1] = tempBytes;
+    }
+
     // now concatenate
     dfs.concat(trgPath, files);
     
@@ -476,4 +494,74 @@ public class TestHDFSConcat {
     Assert.assertEquals(blockSize * repl * (srcNum + 1),
         summary.getSpaceConsumed());
   }
+
+  @Test
+  public void testConcatRelativeTargetPath() throws IOException {
+    Path dir = new Path("/dir");
+    Path trg = new Path("trg");
+    Path src = new Path(dir, "src");
+    dfs.setWorkingDirectory(dir);
+    DFSTestUtil.createFile(dfs, trg, blockSize, REPL_FACTOR, 1);
+    DFSTestUtil.createFile(dfs, src, blockSize, REPL_FACTOR, 1);
+    dfs.concat(trg, new Path[]{src});
+    assertEquals(blockSize * 2, dfs.getFileStatus(trg).getLen());
+    assertFalse(dfs.exists(src));
+  }
+
+  @Test(timeout = 30000)
+  public void testConcatReservedRelativePaths() throws IOException {
+    String testPathDir = "/.reserved/raw/ezone";
+    Path dir = new Path(testPathDir);
+    dfs.mkdirs(dir);
+    Path trg = new Path(testPathDir, "trg");
+    Path src = new Path(testPathDir, "src");
+    DFSTestUtil.createFile(dfs, trg, blockSize, REPL_FACTOR, 1);
+    DFSTestUtil.createFile(dfs, src, blockSize, REPL_FACTOR, 1);
+    try {
+      dfs.concat(trg, new Path[] { src });
+      Assert.fail("Must throw Exception!");
+    } catch (IOException e) {
+      String errMsg = "Concat operation doesn't support "
+          + FSDirectory.DOT_RESERVED_STRING + " relative path : " + trg;
+      GenericTestUtils.assertExceptionContains(errMsg, e);
+    }
+  }
+
+  /**
+   * Test concat on same source and target file which is a inode reference.
+   */
+  @Test
+  public void testConcatOnSameFile() throws Exception {
+    String dir = "/dir1";
+    Path trgDir = new Path(dir);
+    dfs.mkdirs(new Path(dir));
+
+    // create a source file
+    String dir2 = "/dir2";
+    Path srcDir = new Path(dir2);
+    dfs.mkdirs(srcDir);
+    dfs.allowSnapshot(srcDir);
+    Path src = new Path(srcDir, "file1");
+    DFSTestUtil.createFile(dfs, src, 512, (short) 2, 0);
+
+    // make the file as an Inode reference and delete the reference
+    dfs.createSnapshot(srcDir, "s1");
+    dfs.rename(src, trgDir);
+    dfs.deleteSnapshot(srcDir, "s1");
+    Path[] srcs = new Path[1];
+    srcs[0] = new Path(dir, "file1");
+
+    // perform concat
+    LambdaTestUtils.intercept(RemoteException.class,
+        "concat: the src file /dir1/file1 is the same with the target"
+            + " file /dir1/file1",
+        () -> dfs.concat(srcs[0], srcs));
+
+    // the file should exists and read the file
+    byte[] buff = new byte[1080];
+    FSDataInputStream stream = dfs.open(srcs[0]);
+    stream.readFully(0, buff, 0, 512);
+
+    assertEquals(1, dfs.getContentSummary(new Path(dir)).getFileCount());
+  }
 }
