@@ -60,6 +60,9 @@ import org.apache.hadoop.yarn.api.records.ResourceInformation;
 import org.apache.hadoop.yarn.api.records.ResourceRequest;
 import org.apache.hadoop.yarn.api.records.UpdateContainerRequest;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
+import org.apache.hadoop.yarn.event.Dispatcher;
+import org.apache.hadoop.yarn.event.DrainDispatcher;
+import org.apache.hadoop.yarn.event.Event;
 import org.apache.hadoop.yarn.exceptions.ApplicationMasterNotRegisteredException;
 import org.apache.hadoop.yarn.exceptions.InvalidContainerReleaseException;
 import org.apache.hadoop.yarn.exceptions.InvalidResourceRequestException;
@@ -70,6 +73,7 @@ import org.apache.hadoop.yarn.security.ContainerTokenIdentifier;
 import org.apache.hadoop.yarn.server.resourcemanager.resource.TestResourceProfiles;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.RMApp;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.attempt.RMAppAttempt;
+import org.apache.hadoop.yarn.server.resourcemanager.rmapp.attempt.RMAppAttemptEventType;
 import org.apache.hadoop.yarn.server.resourcemanager.rmapp.attempt.RMAppAttemptState;
 import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainer;
 import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainerEvent;
@@ -78,7 +82,6 @@ import org.apache.hadoop.yarn.server.resourcemanager.scheduler.ResourceScheduler
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacityScheduler;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.LeafQueue;
-import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.TestUtils;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.FairScheduler;
 
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair
@@ -742,8 +745,8 @@ public class TestApplicationMasterService {
     MockRM rm = new MockRM(yarnConf);
     rm.start();
 
-    MockNM nm1 = rm.registerNode("199.99.99.1:1234", TestUtils
-        .createResource(DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_MB,
+    MockNM nm1 = rm.registerNode("199.99.99.1:1234", ResourceTypesTestHelper
+        .newResource(DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_MB,
             DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_VCORES, null));
 
     RMApp app1 = rm.submitApp(GB, "app", "user", null, "default");
@@ -890,10 +893,10 @@ public class TestApplicationMasterService {
     CapacityScheduler cs = (CapacityScheduler) rm.getResourceScheduler();
     LeafQueue leafQueue = (LeafQueue) cs.getQueue("default");
 
-    MockNM nm1 = rm.registerNode("199.99.99.1:1234", TestUtils
-        .createResource(DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_MB,
+    MockNM nm1 = rm.registerNode("199.99.99.1:1234", ResourceTypesTestHelper
+        .newResource(DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_MB,
             DEFAULT_RM_SCHEDULER_MAXIMUM_ALLOCATION_VCORES,
-            ImmutableMap.of("res_1", 4)));
+            ImmutableMap.of("res_1", "4")));
 
     RMApp app1 = rm.submitApp(GB, "app", "user", null, "default");
     MockAM am1 = MockRM.launchAndRegisterAM(app1, rm, nm1);
@@ -905,8 +908,8 @@ public class TestApplicationMasterService {
     boolean exception = false;
     try {
       am1.allocate(Collections.singletonList(ResourceRequest.newBuilder()
-              .capability(TestUtils.createResource(9 * GB, 1,
-                      ImmutableMap.of("res_1", 1)))
+              .capability(ResourceTypesTestHelper.newResource(9 * GB, 1,
+                      ImmutableMap.of("res_1", "1")))
               .numContainers(1)
               .resourceName("*")
               .build()), null);
@@ -920,7 +923,8 @@ public class TestApplicationMasterService {
       // Now request resource, vcores > allowed
       am1.allocate(Collections.singletonList(ResourceRequest.newBuilder()
           .capability(
-              TestUtils.createResource(8 * GB, 18, ImmutableMap.of("res_1", 1)))
+              ResourceTypesTestHelper.newResource(8 * GB, 18,
+                  ImmutableMap.of("res_1", "1")))
               .numContainers(1)
               .resourceName("*")
               .build()), null);
@@ -933,8 +937,8 @@ public class TestApplicationMasterService {
     try {
       // Now request resource, res_1 > allowed
       am1.allocate(Collections.singletonList(ResourceRequest.newBuilder()
-              .capability(TestUtils.createResource(8 * GB, 1,
-                      ImmutableMap.of("res_1", 100)))
+              .capability(ResourceTypesTestHelper.newResource(8 * GB, 1,
+                      ImmutableMap.of("res_1", "100")))
               .numContainers(1)
               .resourceName("*")
               .build()), null);
@@ -990,4 +994,54 @@ public class TestApplicationMasterService {
         app1.getApplicationId()).getOriginalTrackingUrl());
     rm.stop();
   }
+
+  @Test(timeout = 120000)
+  public void testRepeatedFinishApplicationMaster() throws Exception {
+
+    CountingDispatcher dispatcher = new CountingDispatcher();
+    MockRM rm = new MockRM(conf) {
+      @Override
+      protected Dispatcher createDispatcher() {
+        return dispatcher;
+      }
+    };
+
+    try {
+      rm.start();
+      // Register node1
+      MockNM nm1 = rm.registerNode("127.0.0.1:1234", 6 * GB);
+      // Submit an application
+      RMApp app1 = rm.submitApp(2048);
+      MockAM am1 = MockRM.launchAM(app1, rm, nm1);
+      am1.registerAppAttempt();
+      FinishApplicationMasterRequest req = FinishApplicationMasterRequest
+          .newInstance(FinalApplicationStatus.FAILED, "", "");
+      for (int i = 0; i < 10; i++) {
+        am1.unregisterAppAttempt(req, false);
+      }
+      Assert.assertEquals("Expecting only one event", 1,
+          dispatcher.getEventCount());
+    } finally {
+      rm.stop();
+    }
+  }
+
+  static class CountingDispatcher extends DrainDispatcher {
+    private int eventreceived = 0;
+
+    @SuppressWarnings("rawtypes")
+    @Override
+    protected void dispatch(Event event) {
+      if (event.getType() == RMAppAttemptEventType.UNREGISTERED) {
+        eventreceived++;
+      } else {
+        super.dispatch(event);
+      }
+    }
+
+    public int getEventCount() {
+      return eventreceived;
+    }
+  }
+
 }
